﻿'@copyright  	Copyright (c) 2014 Toradex AG.
'               [Software License Agreement]
'$Author:		khv $
'$Date: 		$ 2014-06-18 $
'@brief:		Registry access demo in VB.NET.

Public Class Form1

    Const MAX_BUFFER_SIZE As UInteger = 256

    Const REG_SZ As UInteger = &H1
    Const REG_BINARY As UInteger = &H3
    Const REG_DWORD As UInteger = &H4
    Const REG_MULTI_SZ As UInteger = &H7

    Const KEY_READ As UInteger = &H20019
    Const KEY_WRITE As UInteger = &H20006
    Const KEY_ALL_ACCESS As UInteger = &HFF003F
    Const ERROR_SUCCESS As UInteger = &H0
    Const REG_OPTION_VOLATILE As UInteger = &H1
    Const REG_OPTION_NON_VOLATILE As UInteger = &H0

    Dim HKEY_CLASSES_ROOT As UIntPtr = CUInt(&H80000000L)
    Dim HKEY_CURRENT_USER As UIntPtr = CUInt(&H80000001L)
    Dim HKEY_LOCAL_MACHINE As UIntPtr = CUInt(&H80000002L)
    Dim HKEY_USERS As UIntPtr = CUInt(&H80000003L)

    Dim operationToPerform As String = ""
    Dim mainKeySelect As UIntPtr = UIntPtr.Zero
    Dim subKeyPath As String = ""
    Dim keyValueName As String = ""
    Dim retVal As UInteger = 0
    Dim retOpenKey As UIntPtr = UIntPtr.Zero
    Dim retNewKey As UIntPtr = UIntPtr.Zero
    Dim dispositionType As UInteger = 0
    Dim writeRegData(MAX_BUFFER_SIZE) As Byte
    Dim readRegData(MAX_BUFFER_SIZE) As Byte
    Dim readBufferLength As UInteger
    Dim dataInWriteBuffer As UInteger
    Dim dataType As UInteger = &H0
    Dim allKeysClosed As Boolean = False

    ' Shows error code occurred in any procedure.
    Public Function ShowErrorCode(ByVal errorCode As UInteger)
        Select Case errorCode
            Case 2
                If operationToPerform = "Read a value" Or operationToPerform = "Delete a value" Then
                    MessageBox.Show("Value not found.", "Error")
                Else
                    MessageBox.Show("Key not found.", "Error")
                End If
            Case 5
                MessageBox.Show("Access denied.", "Error")
            Case 87
                MessageBox.Show("Invalid parameter.", "Error")
            Case 234
                MessageBox.Show("Data too large to handle.", "Error")
            Case Else
                MessageBox.Show("Error Code (winerror.h) = " + errorCode, "Error")
        End Select
        Return Nothing
    End Function

    ' Converts an unsigned integer 32 bit value to byte array.
    Public Function ConvertIntToBytes(ByVal intValue As UInteger) As Byte()
        Dim byteArray(4) As Byte
        Dim count As Integer = 3

        While count >= 0
            byteArray(count) = intValue Mod 256
            intValue >>= 8
            count -= 1
        End While
        Return byteArray
    End Function

    ' Takes data in string format (as in textbox) and converts that to byte array i.e. in registry writeable form.
    Public Function GetBytesAndLength(ByVal sourceString As String, ByVal valueType As UInteger) As Byte()
        Dim retBytes(MAX_BUFFER_SIZE) As Byte
        Dim data As UInteger

        Select Case valueType
            Case REG_SZ, REG_MULTI_SZ
                retBytes = System.Text.Encoding.Unicode.GetBytes(sourceString)
                dataInWriteBuffer = sourceString.Length * 2       ' Wide character length = Normal character length * 2.
            Case REG_DWORD
                data = Convert.ToUInt32(sourceString)             ' String to DWORD conversion.
                dataInWriteBuffer = 4                             ' Unsigned int = 4 bytes.
                retBytes = BitConverter.GetBytes(data)            ' Unsigned int to byte array conversion.
            Case REG_BINARY
                data = Convert.ToUInt32(sourceString)             ' String to unsigned int conversion.
                dataInWriteBuffer = 4                             ' Unsigned int = 4 bytes.
                retBytes = ConvertIntToBytes(data)                ' Separate 4 bytes of unsigned int.
        End Select
        Return retBytes
    End Function

    ' Shows data received by "Read a value" operation to the user.
    Public Function ShowReadData(ByVal dataBytes() As Byte, ByVal dataType As UInteger, ByVal dataLength As UInteger)
        Dim byteArrayToInt As UInteger
        Dim totalBytesToPrint As UInteger = 0
        Dim dataCopied As UInteger = 0
        Dim count As UInteger = 1

        Select Case dataType
            Case REG_SZ
                ValueType.SelectedItem = "String"
                Value.Text = System.Text.Encoding.Unicode.GetString(dataBytes, 0, dataLength)
            Case REG_DWORD
                ValueType.SelectedItem = "Dword"
                byteArrayToInt = BitConverter.ToUInt32(dataBytes, 0)          ' Byte array to unsigned int conversion.
                Value.Text = byteArrayToInt.ToString()                        ' Unsigned int to string conversion.
            Case REG_BINARY
                ValueType.SelectedItem = "Binary"
                Array.Reverse(dataBytes, 0, 4)                                ' Reverse all 4 bytes in byte array.
                byteArrayToInt = BitConverter.ToUInt32(dataBytes, 0)          ' Byte array to unsigned int conversion.
                Value.Text = byteArrayToInt.ToString()
            Case REG_MULTI_SZ
                ValueType.SelectedItem = "Multi String"
                Value.Enabled = True
                Value.Text = System.Text.Encoding.Unicode.GetString(dataBytes, 0, dataLength)
                While totalBytesToPrint < dataLength                          ' Print each string from byte array 
                    Value.Text += Environment.NewLine
                    totalBytesToPrint = Value.Text.Length * 2
                    dataCopied = totalBytesToPrint - count
                    Value.Text += System.Text.Encoding.Unicode.GetString(dataBytes, dataCopied, dataLength - dataCopied)
                    count += 2
                End While
        End Select
        Return Nothing
    End Function

    ' Verifies the presence of data in required fields to perform the selected operation.
    Public Function VerifyRequest() As Boolean
        If operationToPerform = "" Then
            MessageBox.Show("Select operation to perform.", "Error")
            Return False
        End If
        If mainKeySelect = UIntPtr.Zero Then
            MessageBox.Show("Select main key.", "Error")
            Return False
        End If
        Select Case operationToPerform
            Case "Create new key", "Delete a key", "Read a value", "Delete a value"
                If NewKeyValue.Text = "" Then
                    MessageBox.Show("Enter name of the key/value.", "Error")
                    Return False
                End If
            Case "Create new value", "Modify a value"
                If NewKeyValue.Text = "" Then
                    MessageBox.Show("Enter name of the value.", "Error")
                    Return False
                End If
                If dataType = 0 Then
                    MessageBox.Show("Enter value type.", "Error")
                    Return False
                End If
                If Value.Text.Length > 127 Then
                    MessageBox.Show("Value too large.", "Error")
                    Return False
                End If
                If ValueType.SelectedItem = "Binary" Or ValueType.SelectedItem = "Dword" Then
                    Try
                        Convert.ToUInt32(Value.Text)
                    Catch ex As Exception
                        MessageBox.Show("Enter an integer value.", "Error")
                        Return False
                    End Try
                End If
        End Select
        Return True
    End Function

    Private Sub MainSelection_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MainSelection.SelectedIndexChanged
        If MainSelection.Text = "Create new key" Then
            operationToPerform = "Create new key"
            ValueType.Enabled = False
            Value.Enabled = False
        ElseIf MainSelection.SelectedItem = "Delete a key" Then
            operationToPerform = "Delete a key"
            ValueType.Enabled = False
            Value.Enabled = False
        ElseIf MainSelection.SelectedItem = "Create new value" Then
            operationToPerform = "Create new value"
            ValueType.Enabled = True
            Value.Enabled = True
        ElseIf MainSelection.SelectedItem = "Read a value" Then
            operationToPerform = "Read a value"
            ValueLabel.Text = "Value"
            Value.Text = ""
            ValueType.Enabled = False
            Value.Enabled = False
        ElseIf MainSelection.SelectedItem = "Modify a value" Then
            operationToPerform = "Modify a value"
            ValueType.Enabled = True
            Value.Enabled = True
        ElseIf MainSelection.SelectedItem = "Delete a value" Then
            operationToPerform = "Delete a value"
            ValueType.Enabled = False
            Value.Enabled = False
        End If
    End Sub

    Private Sub ExecuteCommand_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ExecuteCommand.Click
        Dim verify As Boolean = VerifyRequest()
        If verify = False Then
            Return
        End If

        subKeyPath = SubKey.Text                                    ' Gets sub key name from sub key textbox.
        keyValueName = NewKeyValue.Text                             ' Gets key/value name from key/value textbox.

        Select Case operationToPerform
            Case "Create new key"
                retVal = registry.RegOpenKeyEx(mainKeySelect, subKeyPath, UIntPtr.Zero, KEY_ALL_ACCESS, retOpenKey)         ' Opens the key.
                If (retVal <> ERROR_SUCCESS) Then
                    ShowErrorCode(retVal)
                Else
                    retVal = ERROR_SUCCESS
                    allKeysClosed = False
                    retVal = registry.RegCreateKeyEx(retOpenKey, keyValueName, 0, "", REG_OPTION_NON_VOLATILE, 0, UIntPtr.Zero, retNewKey, dispositionType)     ' Creates new key.
                    If (retVal <> ERROR_SUCCESS) Then
                        ShowErrorCode(retVal)
                    Else
                        MessageBox.Show("  Key created.", "Success")
                    End If
                End If
                registry.RegCloseKey(retNewKey)           ' Closes the open key.
                registry.RegCloseKey(retOpenKey)          ' Closes the open key.
                allKeysClosed = True

            Case "Delete a key"
                retVal = registry.RegOpenKeyEx(mainKeySelect, subKeyPath, UIntPtr.Zero, KEY_ALL_ACCESS, retOpenKey)         ' Opens the key.
                If (retVal <> ERROR_SUCCESS) Then
                    ShowErrorCode(retVal)
                Else
                    retVal = ERROR_SUCCESS
                    allKeysClosed = False
                    retVal = registry.RegDeleteKey(retOpenKey, keyValueName)                ' Deletes the key.
                    If (retVal <> ERROR_SUCCESS) Then
                        ShowErrorCode(retVal)
                    Else
                        MessageBox.Show("  Key deleted.", "Success")
                    End If
                End If
                registry.RegCloseKey(retOpenKey)          ' Closes the open key.
                allKeysClosed = True

            Case "Create new value", "Modify a value"
                retVal = registry.RegOpenKeyEx(mainKeySelect, subKeyPath, UIntPtr.Zero, KEY_ALL_ACCESS, retOpenKey)         ' Opens the key.
                If (retVal <> ERROR_SUCCESS) Then
                    ShowErrorCode(retVal)
                Else
                    retVal = ERROR_SUCCESS
                    allKeysClosed = False
                    writeRegData = GetBytesAndLength(Value.Text, dataType)      ' Converts data given by user to registry writeable from.
                    retVal = registry.RegSetValueEx(retOpenKey, keyValueName, UIntPtr.Zero, dataType, writeRegData, dataInWriteBuffer)      ' Creates/Modifies a value.
                    If (retVal <> ERROR_SUCCESS) Then
                        ShowErrorCode(retVal)
                    Else
                        MessageBox.Show("Modification done.", "Success")
                    End If
                End If
                registry.RegCloseKey(retOpenKey)          ' Closes the open key.
                allKeysClosed = True

            Case "Read a value"
                retVal = registry.RegOpenKeyEx(mainKeySelect, subKeyPath, UIntPtr.Zero, KEY_ALL_ACCESS, retOpenKey)          ' Opens the key.
                If (retVal <> ERROR_SUCCESS) Then
                    ShowErrorCode(retVal)
                Else
                    retVal = ERROR_SUCCESS
                    allKeysClosed = False
                    readBufferLength = MAX_BUFFER_SIZE    ' Size of buffer to pass to RegQueryValueEx()
                    retVal = registry.RegQueryValueEx(retOpenKey, keyValueName, 0, dataType, readRegData, readBufferLength)     ' Reads a value.
                    If (retVal <> ERROR_SUCCESS) Then
                        ShowErrorCode(retVal)
                    Else
                        ShowReadData(readRegData, dataType, readBufferLength)
                    End If
                End If
                registry.RegCloseKey(retOpenKey)          ' Closes the open key.
                allKeysClosed = True

            Case "Delete a value"
                retVal = registry.RegOpenKeyEx(mainKeySelect, subKeyPath, UIntPtr.Zero, KEY_ALL_ACCESS, retOpenKey)         ' Opens the key.
                If (retVal <> ERROR_SUCCESS) Then
                    ShowErrorCode(retVal)
                Else
                    retVal = ERROR_SUCCESS
                    allKeysClosed = False
                    retVal = registry.RegDeleteValue(retOpenKey, keyValueName)          ' Deletes a value.
                    If (retVal <> ERROR_SUCCESS) Then
                        ShowErrorCode(retVal)
                    Else
                        MessageBox.Show("Value deleted.", "Success")
                    End If
                End If
                registry.RegCloseKey(retOpenKey)        ' Closes the open key.
                allKeysClosed = True
        End Select
    End Sub

    Private Sub MainKey_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MainKey.SelectedIndexChanged
        If MainKey.Text = "HKEY_CLASSES_ROOT" Then
            mainKeySelect = HKEY_CLASSES_ROOT
        ElseIf MainKey.Text = "HKEY_CURRENT_USER" Then
            mainKeySelect = HKEY_CURRENT_USER
        ElseIf MainKey.Text = "HKEY_LOCAL_MACHINE" Then
            mainKeySelect = HKEY_LOCAL_MACHINE
        ElseIf MainKey.Text = "HKEY_USERS" Then
            mainKeySelect = HKEY_USERS
        End If
    End Sub
    
    Private Sub ValueType_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ValueType.SelectedIndexChanged
        If ValueType.SelectedItem = "String" Then
            dataType = REG_SZ
            ValueLabel.Text = "Value (String)"
            Value.Multiline = False
        ElseIf ValueType.SelectedItem = "Dword" Then
            dataType = REG_DWORD
            ValueLabel.Text = "Value (Decimal)"
            Value.Multiline = False
        ElseIf ValueType.SelectedItem = "Binary" Then
            dataType = REG_BINARY
            ValueLabel.Text = "Value (Decimal)"
            Value.Multiline = False
        ElseIf ValueType.SelectedItem = "Multi String" Then
            dataType = REG_MULTI_SZ
            ValueLabel.Text = "Value (Multi String)"
            Value.Multiline = True
        End If
        Value.Text = ""
    End Sub

    Private Sub SaveReg_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles SaveReg.Click
        If mainKeySelect = UIntPtr.Zero Then
            MessageBox.Show("Select main key.", "Error")
        Else
            retVal = registry.RegFlushKey(mainKeySelect)
            If retVal <> ERROR_SUCCESS Then
                MessageBox.Show("Error Code = " + retVal, "Error")
            Else
                MessageBox.Show("Main key saved.", "Success")
            End If
        End If
    End Sub

    Private Sub Form1_Closing(ByVal sender As Object, ByVal e As System.ComponentModel.CancelEventArgs) Handles MyBase.Closing
        If allKeysClosed = False Then
            registry.RegCloseKey(retNewKey)
            registry.RegCloseKey(retOpenKey)
        End If
    End Sub
End Class